/******************************************************************************
*	BC727X数码管及键盘接口芯片驱动库
*
* 程序说明：BC727x芯片显示及键盘接口的驱动函数库
* 版权所有:	北京凌志比高科技有限公司 https://bitcode.com.cn
* 版本号: V3.2
* 初始版本：2021年3月15日
* 版本历史:
*   2021年3月   V1.0
*   2021年4月   V2.0  display_dec()函数增加了Width参数,修正fifo操作错误,修改函数名称
*                    set_tx_eni_func() -> set_spi_eni_func()
*                    set_tx_disi_func() -> set_spi_disi_func()
*   2021年4月   V2.1  修正上电报告0键长按错误，修正误报长按键错误
*   2021年4月   V2.2  增加clear()函数
*   2021年4月   V2.3  修正clear()函数对闪烁属性清除中的bug,改写格式以兼容C89等较早C语言标准
*   2021年4月   V3.0  移除缓冲区发送完毕时禁止中断代码。修改了部分API函数名称以和UART单线LED
*                    显示驱动库保持一致:
*                      bc727x_send_cmd() ---> send_cmd()
*                      set_spi_wt_func() ---> set_write_func()
*                      set_tx_eni_func() ---> set_eni_func()
*                      set_tx_disi_func() ---> set_disi_func()
*                      spi_tx_buf_ready() ---> tx_ready()
*   2021年4月   V3.1  非中断模式增加CS控制
*   2021年5月   V3.2  display_dec()函数增加显示前导0功能
*   2022年5月   V4.0  增加display_float()函数
*   2023年4月   V4.1  重写了keyboard struct初始化部分以兼容C89标准
*   2024年8月   V5.0  移除了驱动库中所有CS相关的操作和函数，片选完全由用户控制
*                    增加了is_spi_busy()函数（中断模式下）
*
* 使用方法：
*   将此头文件加入所有需要使用本驱动库的c源文件，并将bc727x.c文件加入项目源文件列表，可能还需要
*   将本本件所在路径加入项目的include搜索路径，即可从用户程序中直接调用本库中函数。
*   驱动库的配置通过 bc727x_config.h 文件完成。
*	详细使用方法，参见《BC727X数码管及键盘接口芯片驱动库技术说明书》
******************************************************************************/

#include "bc727x.h"

#ifndef NULL
#define NULL 0
#endif

const uint8_t BlinkBit[] = { 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80 };
const uint32_t Mul[8] = {1, 10, 100, 1000, 10000, 100000, 1000000, 10000000};

/************* 以下为中断模式代码 ****************/
#if SPI_MODE_INTERRUPT == 1

struct display_t
{
    volatile uint8_t FifoFreeSpace;
    volatile uint8_t Busy;
    uint8_t          NextWritePos;
    volatile uint8_t NextReadPos;
    volatile uint8_t OutCnt;
    uint8_t          InCnt;
    uint8_t          BlinkCtlL;
    uint8_t          BlinkCtlH;
#if (SPI_MODE_16BIT == 0)
    uint8_t SpiFifo[SPI_FIFO_SIZE];
    void (*pSpiWriteTxBuf)(uint8_t);
#else
    uint16_t SpiFifo[SPI_FIFO_SIZE];
    void (*pSpiWriteTxBuf)(uint16_t);
#endif
    void (*pDisableSpiTxInt)(void);
    void (*pEnableSpiTxInt)(void);
};

struct display_t Display = {
    .FifoFreeSpace = SPI_FIFO_SIZE
};

/******************************************************************************
* BC727X发送指令 send_cmd()
* 为基本函数，可以发送任何指令到BC727x。其它函数为上层函数，封装显示中常用的显示十进制、十六
* 进制数据，闪烁控制等。上层函数通过调用 bc727x_send_cmd()	完成功能。
* 参数: Cmd  -- 指令(寄存器地址)
*      Data -- 数据
******************************************************************************/
void send_cmd(uint8_t Cmd, uint8_t Data)
{
    while (Display.FifoFreeSpace == 0)
        ; /* 检查FIFO中是否还有空间 */
    if (Display.pDisableSpiTxInt != NULL)
    {
        Display.pDisableSpiTxInt(); /* 禁止SPI中断防止冲突 */
    }
#if (SPI_MODE_16BIT == 0)
    Display.SpiFifo[Display.NextWritePos] = Cmd; /* 写入新数据 */
#else
    Display.SpiFifo[Display.NextWritePos] = (((uint16_t)Cmd) << 8) | Data;
#endif
    Display.NextWritePos = (Display.NextWritePos + 1) & (SPI_FIFO_SIZE - 1); /* 调整写指针到下一单元 */
    --Display.FifoFreeSpace;                                                 /* FIFO空间数-1 */
    if (!Display.Busy)                                                       /*	如果函数调用之前FIFO已经清空(已经停止自动发送) */
    {
        Display.Busy = 1;
        if (Display.pSpiWriteTxBuf != NULL)
        {
            Display.pSpiWriteTxBuf(Display.SpiFifo[Display.NextReadPos]); /*	启动SPI发送 */
            Display.OutCnt++;
        }
        Display.NextReadPos = (Display.NextReadPos + 1) & (SPI_FIFO_SIZE - 1); /* 调整读指针到下一单元 */
        ++Display.FifoFreeSpace;                                               /* 可用空间+1 */
    }
#if (SPI_MODE_16BIT == 0)
    if (Display.pEnableSpiTxInt != NULL)
    {
        Display.pEnableSpiTxInt(); /* 重新使能SPI中断 */
    }
    while (Display.FifoFreeSpace == 0) /* 如果是8位SPI模式，继续将数据字节写入FIFO */
        ;                              /* 检查FIFO中是否还有空间 */
    if (Display.pDisableSpiTxInt != NULL)
    {
        Display.pDisableSpiTxInt(); /* 禁止SPI中断防止冲突 */
    }
    --Display.FifoFreeSpace;                                                                  /*	FIFO空间数-1 */
    Display.SpiFifo[Display.NextWritePos] = Data;                                             /*	写入新数据 */
    Display.NextWritePos                  = (Display.NextWritePos + 1) & (SPI_FIFO_SIZE - 1); /*	调整写指针到下一单元 */
#endif
    if (Display.pEnableSpiTxInt != NULL)
    {
        Display.pEnableSpiTxInt(); /* 重新使能SPI中断 */
    }
}

/******************************************************************************
* SPI发送缓冲区准备好函数	tx_ready()
* 在使用SPI中断模式时，当SPI外设的发送寄存器可以接受下一个数据时，用户调用此函数通知驱动库。
* 一般SPI外设在当前数据(字节或word)发送完成可以接受下一个数据时，会产生一个中断，用户在这个
* 中断处理程序中呼叫此函数通知驱动库发送下一个数据到SPI外设。
******************************************************************************/
void tx_ready(void)
{
    if (Display.FifoFreeSpace < SPI_FIFO_SIZE) /* 如果FIFO尚未全部发送 */
    {
        if (Display.pSpiWriteTxBuf != NULL)
        {
            Display.pSpiWriteTxBuf(Display.SpiFifo[Display.NextReadPos]); /* 发送下一字符 */
            Display.OutCnt++;
        }
        Display.NextReadPos = (Display.NextReadPos + 1) & (SPI_FIFO_SIZE - 1);
        ++Display.FifoFreeSpace; /*	可用空间+1 */
    }
}

/******************************************************************************
* SPI检查是否发送结束 spi_check_end()
* 
* 当使用中断模式时用户必须在SPI接收中断中，呼叫此函数，令驱动库获取串口缓冲区是否所有数据
* 均已从SPI口发送出去的信息。
******************************************************************************/
void spi_check_end(void)
{
    Display.InCnt++;
    if ((Display.FifoFreeSpace == SPI_FIFO_SIZE) && (Display.OutCnt == Display.InCnt))
    {
    	Display.Busy = 0;
    }
}


/******************************************************************************
* 使用中断时查询SPI是否在发送中函数 is_spi_busy()
* 在使用SPI中断模式时，驱动库将待发送数据写入缓冲区后，会立即返回，主程序可继续执行后续操作，
* 而SPI数据发送则交由后台进行。因为片选信号完全由用户控制，因此有可能存在SPI缓冲区还还有未发
* 送数据时用户即将片选CS置高造成芯片接收不到后半部分数据的风险。因此我们希望用户在将CS信号
* 置为高电平之前，先使用此函数查询SPI的状态，只有在SPI不忙时，才可将CS置高。
* 参数： 无
* 返回值： 0 --- SPI空闲
*        1 --- SPI数据发送中
******************************************************************************/
uint8_t is_spi_busy(void)
{
	return Display.Busy;
}

/******************************************************************************
* 设置(注册)SPI发送中断使能/禁止函数 set_eni_func(), set_disi_func()
* 用户通过调用此函数告知驱动库调用哪个函数完成使能和禁止SPI发送中断。
* 这个调用接口仅在SPI使用中断模式时需要。
* 参数： pTxIntEnFunc   --	指向发送中断使能函数的指针，该函数必须无输入参数，无返回参数。
*	pTxIntDisFunc  -- 指向发送中断禁止函数的指针，该函数必须无输入参数，无返回参数。
******************************************************************************/
void set_eni_func(void (*pTxIntEnFunc)(void))
{
    Display.pEnableSpiTxInt = pTxIntEnFunc;
}

void set_disi_func(void (*pTxIntDisFunc)(void))
{
    Display.pDisableSpiTxInt = pTxIntDisFunc;
}

#else

/************* 以下为非中断模式代码 ****************/
struct display_t
{
    uint8_t BlinkCtlL;
    uint8_t BlinkCtlH;
#if (SPI_MODE_16BIT == 0)
    void (*pSpiWriteTxBuf)(uint8_t);
#else
    void (*pSpiWriteTxBuf)(uint16_t);
#endif
};

struct display_t Display;

/******************************************************************************
* BC727X发送指令 send_cmd()
* 为基本函数，可以发送任何指令到BC727x。其它函数为上层函数，封装显示中常用的显示十进制、十六
* 进制数据，闪烁控制等。上层函数通过调用 bc727x_send_cmd()	完成功能。
* 参数: Cmd  -- 指令(寄存器地址)
*      Data -- 数据
******************************************************************************/
void send_cmd(uint8_t Cmd, uint8_t Data)
{
#if SPI_MODE_16BIT == 0
    Display.pSpiWriteTxBuf(Cmd);
    Display.pSpiWriteTxBuf(Data);
#else
    Display.pSpiWriteTxBuf((((uint16_t)Cmd) << 8) | Data);
#endif
}

#endif

/**************	以下为公共代码	*******************/

/******************************************************************************
* 清除显示 clear()
* 清除所有的显示和闪烁属性
* 参数: 无
******************************************************************************/
void clear(void)
{
    uint8_t i;
    send_cmd(WRITE_ALL, 0xff);
    for (i = 0; i < 8; i++)
    {
        send_cmd(SEG_BLINK_CTL + i, 0);
    }
    send_cmd(DIG_BLINK_CTL_L, 0);
    send_cmd(DIG_BLINK_CTL_H, 0);
    Display.BlinkCtlL = 0;
    Display.BlinkCtlH = 0;
}

/******************************************************************************
* 以16进制显示数值 display_hex()
* 假设数码管显示位按从低到高从右向左排列，此函数将Val按16进制显示，最低位在第Pos位数码管，第2
* 位显示在Pos+1位,依次类推，最大显示数值为 0xFFFF
* 参数: Val    --	待显示数值，范围 0 - 0xFFFF
*      Pos    -- 最低位显示位置，范围 0 -	15, 如果数值超出所用芯片显示位数，则超出部分无显示
*      Width  -- 16进制显示位数。本函数最大显示4位16进制数字，但可以设置大于或小于4位的显示，
*		 如果设置超过4位的位数，超出的显示位将显示0
******************************************************************************/
void display_hex(uint16_t Val, uint8_t Pos, uint8_t Width)
{
    int8_t i;
    if ((Pos > 15) || (Width == 0))
    {
        return;
    }
#if LOW_DIG_NUM_ON_RIGHT == 1
    if (Width + Pos > 16)
    {
        Width = 16 - Pos;
    }
    for (i = Pos; i < Width + Pos; i++)
    {
        send_cmd(DECODE_HEX, (i << 4) | (Val & 0x0f));
        Val >>= 4;
    }
#else
    if (Width > Pos + 1)
    {
        Width = Pos + 1;
    }
    for (i = Pos; i > (Pos - Width); i--)
    {
        send_cmd(DECODE_HEX, (i << 4) | (Val & 0x0f));
        Val >>= 4;
    }
#endif
}

/******************************************************************************
* 以10进制显示数值 display_dec()
* 假设数码管显示位按从低到高从右向左排列，此函数将Val按10进制显示，个位在第Pos位数码管，十
* 位显示在Pos+1位,依次类推，最大显示数值为 4294967295
* 参数: Val    --	待显示数值，范围 0 - 0xFFFFFFFF
*      Pos    -- 最低位显示位置，范围 0 -	15, 如果数值超出所用芯片显示位数，则超出部分无显示
*      Width  -- 从最低位开始算起的数值显示的宽度,最高位控制是否显示前导0,bit7=1显示前导0
******************************************************************************/
void display_dec(uint32_t Val, uint8_t Pos, uint8_t Width)
{
    if ((Pos > 15) || ((Width & 0x7f) == 0))
    {
        return;
    }
    send_cmd(DECODE_HEX, (Pos << 4) | (Val % 10));
    Val = Val / 10;
#if LOW_DIG_NUM_ON_RIGHT == 1
    if (++Pos > 15)
    {
        return;
    }
    while ((--Width & 0x7f) > 0)
    {
        if (Val != 0)
        {
            send_cmd(DECODE_HEX, (Pos << 4) | (Val % 10));
        }
        else
        {
            if (Width & 0x80)
            {
                send_cmd(DECODE_HEX, Pos << 4); /* 如果设置为显示前导0,则显示0 */
            }
            else
            {
                send_cmd(Pos, 0xff); /* 如果设置为不显示前导0, 超过数字宽度的部分显示空白 */
            }
        }
        Val = Val / 10;
        if (++Pos > 15)
        {
            break;
        }
    }
#else
    if (Pos-- == 0)
    {
        return;
    }
    while ((--Width & 0x7f) > 0)
    {
        if (Val != 0)
        {
            send_cmd(DECODE_HEX, (Pos << 4) | (Val % 10));
        }
        else
        {
            if (Width & 0x80)
            {
                send_cmd(DECODE_HEX, Pos << 4);
            }
            else
            {
                send_cmd(Pos, 0xff);
            }
        }
        Val = Val / 10;
        if (Pos-- == 0)
        {
            break;
        }
    }
#endif
}

/******************************************************************************
* 显示浮点数 display_float()
* 以十进制小数的方式显示浮点数，但不显示小数点和正负号。
* 参数: Val    -- 待显示浮点数
*      Pos    -- 最低位显示位置，即小数点后最低位的位置，使用方法同display_dec()中对应参数
*      Width  -- 从最低位开始算起的数值显示的宽度,包括小数部分和整数部分，其它同display_dec()
*      Frac   -- 小数点后的位数
******************************************************************************/
void display_float(float Val, uint8_t Pos, uint8_t Width, uint8_t Frac)
{
	uint32_t IntVal;
	if (((Width&0x7f) >= Frac) && (Frac <8))
	{
		if (Val < 0)
		{
			Val = -Val;
		}
		IntVal = (uint32_t)(Val*Mul[Frac]+0.5);
		display_dec(IntVal, Pos, Frac|0x80);
#if LOW_DIG_NUM_ON_RIGHT == 1
		display_dec(IntVal/Mul[Frac], Pos+Frac, Width-Frac);
#else
		display_dec(IntVal/Mul[Frac], Pos-Frac, Width-Frac);
#endif
	}
}


/******************************************************************************
* 位闪烁控制	digit_blink()
* 设置数码管按位闪烁
* 参数: Digit -- 待控制的数码管位，范围 0 - 15
*      OnOff --	闪烁开关，0表示停止闪烁，其它值表示闪烁
******************************************************************************/
void digit_blink(uint8_t Digit, uint8_t OnOff)
{
    if (Digit > 15)
    {
        return;
    }
    if (Digit & 0x08)
    {
        Display.BlinkCtlH = (OnOff) ? Display.BlinkCtlH | BlinkBit[Digit & 0x07] : Display.BlinkCtlH & (~BlinkBit[Digit & 0x07]);
        send_cmd(DIG_BLINK_CTL_H, Display.BlinkCtlH);
    }
    else
    {
        Display.BlinkCtlL = (OnOff) ? Display.BlinkCtlL | BlinkBit[Digit & 0x07] : Display.BlinkCtlL & (~BlinkBit[Digit & 0x07]);
        send_cmd(DIG_BLINK_CTL_L, Display.BlinkCtlL);
    }
}

/******************************************************************************
* 设置(注册)写SPI口函数	set_write_func()
* 用户通过调用此函数告知驱动库调用哪个函数完成向SPI口写入数据的动作。
* 参数： pSpiWriteFunc  --	指向SPI写入函数的指针，该函数必须具有1个输入参数，无返回参数。
*			  根据SPI的工作模式，输入参数须为对应的8位或16位数据。
******************************************************************************/
#if SPI_MODE_16BIT == 0
void set_write_func(void (*pSpiWriteFunc)(uint8_t))
{
    Display.pSpiWriteTxBuf = pSpiWriteFunc;
}
#else
void set_write_func(void (*pSpiWriteFunc)(uint16_t))
{
    Display.pSpiWriteTxBuf = pSpiWriteFunc;
}
#endif


/*********** 以下为键盘相关代码 **********************/
#if USE_KEY_SCAN == 1

/* 定义键盘接口数据结构并设置默认值。此处设置为不检测按键释放，长按键时长为2000次计数 */
struct keyboard_t
{
    volatile uint8_t NewKeyAvailable;
    uint8_t          DetectKeyRelease;
    uint16_t         LastKeyMap;
    uint16_t         ReportedKeyMap;
    volatile uint8_t ReportedKey;
    uint8_t          CombinedKeyCount;
    uint8_t          LongPressKeyCount;
    uint16_t         LongPressCount;
    uint16_t         TimeCounter;
    const uint16_t*  pCombinedKeys;
    const uint8_t*   pLongPressKeys;
    void (*pCallback)(uint8_t);
};

struct keyboard_t Keyboard = {
    0,     /* NewKeyAvailable */
    0,     /* DetectKeyRelease */
    0xFFFF,/* LastKeyMap */
    0xFF,  /* ReportedKey */
    0,     /* CombinedKeyCount */
    0,     /* LongPressKeyCount */
    40,    /* LongPressCount */
    0,     /* TimeCounter */
    NULL,  /* pCombinedKeys */
    NULL,  /* pLongPressKeys */
    NULL   /* pCallback */
};

/******************************************************************************
* 查询是否有新按键
* 根据工作模式的设置，返回是否当前有未读取的新按键。只有符合当前工作模式设置的键盘变化才会返回，
* 比如设置了不检测按键的释放，则按键释放不会改变这个查询的结果。
* 返回值: 如果没有新按键返回0,其它值表示有新按键
******************************************************************************/
uint8_t is_key_changed(void)
{
    return Keyboard.NewKeyAvailable;
}

/******************************************************************************
* 设置键盘检测模式
* 决定是否检测按键(包括组合键)的释放。
* 参数: Mode  --	工作模式， 0=只检测按键按下(导通), 1=同时检测按键按下和释放
******************************************************************************/
void set_detect_mode(uint8_t Mode)
{
    Keyboard.DetectKeyRelease = Mode;
}

/******************************************************************************
* 设置长按键计数值 set_longpress_count()
* 长按键的时间由此计数值决定。用户定期呼叫 long_press_tick() 函数，每次呼叫内部计数器被
* 加一，而每次键盘活动则使计数器清零。当内部计数器的值大于此值时，即触发一个长按键检测，如果
* 此前最后一次按键(包括组合键)属于需检测长按键的键值，则报告一个长按键(长按键的键值是原键值
* 加0x20)
* 参数: CountLimit  --  最大值65534,	达到触发条件所需的计数值，默认值为 40
******************************************************************************/
void set_longpress_count(uint16_t CountLimit)
{
    Keyboard.LongPressCount = CountLimit;
}

/******************************************************************************
* 设置回调函数
* 一般应用时在串口中断中呼叫更新键盘状态函数，而实际的键盘处理程序则放在主循环中。因为处理键盘
* 事件一般不是高优先级的操作，在主程序中处理按键可以减少串口中断占用的处理器时间。当键盘事件
* 需要立刻处理，或者程序为中断驱动型，不存在主循环时，可以设置此回调函数，此函数会在检测到有效
* 键盘事件后自动呼叫。
* 此函数应该具有一个 uint8_t 类型的参数用以接收键值，而返回类型应该为 void. 
* 如果不设置或者设置为 NULL ,则回调函数不会被呼叫。请注意如果 update_key_status()	函数
* 是从串口中断中呼叫的，则此回调函数占用的将是中断的处理时间，与串口中断同级或优先级更低的中断
* 在返回前也不会得到执行，因此回调函数执行不宜占用过长的时间。
* 参数: pCallbackFunc  --	 回调函数的函数指针
 ******************************************************************************/
void set_callback(void (*pCallbackFunc)(uint8_t))
{
    Keyboard.pCallback = pCallbackFunc;
}

uint8_t cal_key_number(uint16_t KeyMap)
{
    uint8_t i;
    for (i = 0; i < 16; i++)
    {
        if (KeyMap & 0x01)
        {
            return i;
        }
        KeyMap >>= 1;
    }
    return 0xff;
}

/******************************************************************************
* 更新键盘状态
* 输入参数为UART所接收数据，此函数一般可在UART中断处理中呼叫，但也可以根据需要从任何地方呼叫。
* 参数: RxData  --  UART从BC6xxx或BC759x所接收到的数据
******************************************************************************/
#if SPI_MODE_16BIT == 1
void update_key_status(uint16_t KeymapData)
#else
void update_key_status(uint8_t Keymap8Bit)
#endif

{
    uint8_t  i;
    uint16_t ChangedKeys;
    uint16_t NewPressedKeys;
    uint16_t TestingCBKey;

#if SPI_MODE_16BIT == 0
    uint16_t       KeymapData;
    static uint8_t ByteCnter = 0;
    static uint8_t MapHighByte;

    if (!(ByteCnter++ & 0x01))
    {
        MapHighByte = Keymap8Bit;
        return;
    }
    KeymapData = (((uint16_t)MapHighByte) << 8) | Keymap8Bit;
#endif

#if SPI_MODE_INTERRUPT == 1
    spi_check_end();
#endif
    ChangedKeys    = Keyboard.LastKeyMap ^ KeymapData;         /* 有变化的位为1 */
    NewPressedKeys = (KeymapData & ChangedKeys) ^ ChangedKeys; /* 获取新按键的按键 */
    if (ChangedKeys != 0)                                      /* 如果键盘映射有变化 */
    {
        Keyboard.TimeCounter = 0;
        if (Keyboard.DetectKeyRelease) /* 如果检测按键释放，或为按键按下，报告新按键 */
        {
            Keyboard.ReportedKey = cal_key_number(ChangedKeys);
            if (KeymapData > Keyboard.LastKeyMap) /* 如果是按键释放 */
            {
                Keyboard.ReportedKey |= 0x80;
            }
            Keyboard.NewKeyAvailable = 1;
            Keyboard.ReportedKeyMap  = KeymapData;
        }
        else if (NewPressedKeys)
        {
            Keyboard.ReportedKey     = cal_key_number(NewPressedKeys);
            Keyboard.NewKeyAvailable = 1;
            Keyboard.ReportedKeyMap  = KeymapData;
        }

        if (Keyboard.CombinedKeyCount != 0) /* 设置了组合键，检查按键是否组合键一部分 */
        {
            for (i = 0; i < Keyboard.CombinedKeyCount; i++) /* 轮询所有组合键 */
            {
                TestingCBKey = *(Keyboard.pCombinedKeys + i);
                if ((KeymapData & TestingCBKey) == 0) /*	组合键所有键均按下 */
                {
                    Keyboard.ReportedKey     = 0x10 + i;
                    Keyboard.NewKeyAvailable = 1;
                    Keyboard.ReportedKeyMap  = KeymapData;
                    break;
                }
                else if (Keyboard.DetectKeyRelease && ((Keyboard.LastKeyMap & TestingCBKey) == 0)) /* 如果之前是按下状态 */
                {
                    Keyboard.ReportedKey     = (0x10 + i) | 0x80;
                    Keyboard.NewKeyAvailable = 1;
                    Keyboard.ReportedKeyMap  = KeymapData;
                    break;
                }
            }
        }

        if (Keyboard.NewKeyAvailable && (Keyboard.pCallback != NULL))
        {
            Keyboard.pCallback(Keyboard.ReportedKey);
            Keyboard.NewKeyAvailable = 0;
        }
    }
    Keyboard.LastKeyMap = KeymapData;
}

/******************************************************************************
* 长按键计数
* 当希望进行长按键检测时，定期呼叫此函数，每呼叫此函数一次，则长按键计数加一。长按键的时间，
* 为用户通过	set_longpress_count() 设置的长按键计数目标值乘以呼叫此函数的时间间隔。此函数
* 一般在定时器中断中呼叫，但也可以根据需要从主程序循环或任何其它地方呼叫。
******************************************************************************/
void long_press_tick(void)
{
    uint8_t i;
    Keyboard.TimeCounter++;

    if (Keyboard.TimeCounter > Keyboard.LongPressCount) /* 如果长按键计时超时 */
    {
        Keyboard.TimeCounter = 0;
        if ((Keyboard.pLongPressKeys != NULL) && (Keyboard.ReportedKeyMap == Keyboard.LastKeyMap))
        {
            for (i = 0; i < Keyboard.LongPressKeyCount; i++) /* 轮询各需要检测的长按键 */
            {
                if (Keyboard.ReportedKey == *(Keyboard.pLongPressKeys + i)) /* 如果最后的按键活动是需要检测长按键的键值之一 */
                {
                    Keyboard.ReportedKey |= 0x20; /*	设置键值为用户定义的长按键键值 */
                    Keyboard.NewKeyAvailable = 1;
                    if (Keyboard.pCallback != NULL)
                    {
                        Keyboard.pCallback(Keyboard.ReportedKey);
                        Keyboard.NewKeyAvailable = 0;
                    }
                    break;
                }
                else if ((Keyboard.LastKeyMap == 0xFFFF) && (*(Keyboard.pLongPressKeys + i) == 0xFF))
                {
                    Keyboard.ReportedKey     = 0xFF;
                    Keyboard.NewKeyAvailable = 1;
                    if (Keyboard.pCallback != NULL)
                    {
                        Keyboard.pCallback(Keyboard.ReportedKey);
                        Keyboard.NewKeyAvailable = 0;
                    }
                    break;
                }
            }
        }
    }
}

/******************************************************************************
* 获取当前键值 get_key_value()
* 驱动库将单按键、组合键和长按键均转化为特定的键值，这样用户可以用统一和简单的逻辑处理各种键盘事
* 件，详细的键值对应关系见说明书。
* 此函数可在任何时候呼叫。呼叫此函数后，如果当前 is_key_changed()	函数返回非零,呼叫后则返
* 回值会变成零。仅返回当前工作模式下有效的键值，即如果设置为不检测按键释放，则仅会返回按键按下所
* 对应的键值。
* 返回值: 键值。
******************************************************************************/
uint8_t get_key_value(void)
{
    Keyboard.NewKeyAvailable = 0;
    return Keyboard.ReportedKey;
}

/******************************************************************************
* 获取当前键盘映射 get_key_map()
* 通过驱动库转换的键值，一次只能反映1个键盘事件，如果有两个键严格同时变化，又不是预定义的组合键
* 时，通过键值将无法得到反映。这种情况或其它特殊需要时，可以通过此函数获取原始的键盘映射，由用户
* 自行处理数据。
* 读取键盘映射不会影响is_key_changed()函数的返回值
* 返回值: 键盘映射
******************************************************************************/
uint16_t get_key_map(void)
{
    return Keyboard.LastKeyMap;
}

/******************************************************************************
* 定义组合键	def_combined_key()
* 此函数通知驱动库组合键的定义。
* 参数: pCBKeyList      --   组合键列表，为一uint16_t数组，数组元素为各个组合键的键盘映
*			                射，为1的位表示该键属于此组合键，当所有1对应的键均按下时，
*			                驱动库即报告一个键盘事件，键值为0x10+此组合键在本列表中的位置
*      CBKeyCount      --   组合键的数量
*
* 欲使用组合键功能，用户需在程序中提供组合键的定义，并通过此函数提交给驱动库。
******************************************************************************/
void def_combined_key(const uint16_t CBKeyList[], uint8_t CBKeyCount)
{
    Keyboard.pCombinedKeys    = CBKeyList;
    Keyboard.CombinedKeyCount = CBKeyCount;
}

/******************************************************************************
* 定义长按键	def_longpress_key()
* 此函数通知驱动库长按键的定义。
* 参数: pLPKeyList      --  长按键列表，为一数组，数组元素为需要检测长按键的键值，可包括
*			               组合键的键值。特殊值0xFF表示检测长时间无按键。
*      LPKeyCount      --  长按键的数量
******************************************************************************/
void def_longpress_key(const uint8_t LPKeyList[], uint8_t LPKeyCount)
{
    Keyboard.pLongPressKeys    = LPKeyList;
    Keyboard.LongPressKeyCount = LPKeyCount;
}

#endif
